=== modified file 'bzr'
--- bzr	2007-01-24 20:40:20 +0000
+++ bzr	2007-02-16 06:06:14 +0000
@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -96,6 +96,16 @@
 
     if profiling:
         profile_imports.log_stack_info(sys.stderr)
-    sys.exit(exit_val)
+
+    # By this point we really have completed everything we want to do, and
+    # there's no point doing any additional cleanup.  Abruptly exiting here
+    # stops any background threads getting into trouble as code is unloaded,
+    # and it may also be slightly faster, through avoiding gc of objects that
+    # are just about to be discarded anyhow.  This does mean that atexit hooks
+    # won't run but we don't use them.  Also file buffers won't be flushed,
+    # but our policy is to always close files from a finally block. -- mbp 20070215
+    sys.stdout.flush()
+    # stderr isn't buffered
+    os._exit(exit_val)
 else:
     pass    # should this give an error? - it can't be used as a lib

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-12-18 15:34:08 +0000
+++ bzrlib/transport/smart.py	2007-02-15 06:15:43 +0000
@@ -867,9 +867,11 @@
 
     def stop_background_thread(self):
         self._should_terminate = True
+        # At one point we would wait to join the threads here, but it looks
+        # like they don't actually exit.  So now we just leave them running
+        # and expect to terminate the process. -- mbp 20070215
         # self._server_socket.close()
-        # we used to join the thread, but it's not really necessary; it will
-        # terminate in time
+        ## sys.stderr.write("waiting for server thread to finish...")
         ## self._server_thread.join()
 
 

